/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2006-2016 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2018 by Naoki Saito
 *             Nagoya Municipal Industrial Research Institute, JAPAN
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: gic_kernel_impl.c 714 2016-03-31 05:52:19Z ertl-hiro $
 */
/*
 *  GIC依存処理（ARM64用）
 */
#include "kernel_impl.h"

/*
 *  CPUインタフェースの操作
 */

/*
 *  CPUインタフェースの初期化
 */
void
gicc_initialize(void)
{
        /*
         *  CPUインタフェースからプロセッサへの割込み信号送出を無効にする
         */
        sil_wrw_mem(GICC_CTLR, GICC_CTLR_DISABLE);

        /*
         *  割込み優先度マスクを最低優先度に設定
         */
        gicc_set_priority((GIC_PRI_LEVEL - 1) << GIC_PRI_SHIFT);

        /*
         *  割込み優先度の全ビット有効に
         */
        sil_wrw_mem(GICC_BPR, 0U);

        /*
         *  アクティブな割込みがあれば，EOIによりクリアする
         */
        sil_wrw_mem(GICC_EOIR, sil_rew_mem(GICC_IAR));

        /*
         *  CPUインタフェースをイネーブル
         */
        sil_wrw_mem(GICC_CTLR, GICC_CTLR_ENABLE);
}

/*
 *  CPUインタフェースの終了処理
 */
void
gicc_terminate(void)
{
        sil_wrw_mem(GICC_CTLR, GICC_CTLR_DISABLE);
}

/*
 *  ディストリビュータの操作
 */

/*
 *  ディストリビュータの初期化
 */
void
gicd_initialize(void)
{
        int             i;

        /*
         *  ディストリビュータを無効にして転送を停止
         */
        sil_wrw_mem(GICD_CTLR, GICD_CTLR_DISABLE);

        /*
         *  すべての割込みの転送を禁止
         */
        for (i = 0; i < (GIC_TNUM_INTNO + 31) / 32; i++) {
                sil_wrw_mem(GICD_ICENABLER(i), 0xffffffffU);
        }

        /*
         *  すべての割込み割込み保留状態をクリア
         */
        for (i = 0; i < (GIC_TNUM_INTNO + 31) / 32; i++) {
                sil_wrw_mem(GICD_ICPENDR(i), 0xffffffffU);
        }

        /*
         *  すべての割込みを最低優先度に設定
         */
        for (i = 0; i < (GIC_TNUM_INTNO + 3) / 4; i++){
                sil_wrw_mem(GICD_IPRIORITYR(i), 0xffffffffU);
        }
        /*
         *  すべての共有ペリフェラル割込みのターゲットをプロセッサ0に設定
         */
        for (i = GIC_INTNO_SPI0 / 4; i < (GIC_TNUM_INTNO + 3) / 4; i++) {
                sil_wrw_mem(GICD_ITARGETSR(i), 0x01010101U);
        }

        /*
         *  すべてのペリフェラル割込みをレベルトリガに設定
         */
        for (i = GIC_INTNO_PPI0 / 16; i < (GIC_TNUM_INTNO + 15) / 16; i++) {
                sil_wrw_mem(GICD_ICFGR(i), 0x00000000U);
        }
        /*
         *  ディストリビュータを有効にして優先度規則による割込みの転送を開始
         */
        sil_wrw_mem(GICD_CTLR, GICD_CTLR_ENABLE);
}

/*
 *  ディストリビュータの終了処理
 */
void
gicd_terminate(void)
{
        sil_wrw_mem(GICD_CTLR, GICD_CTLR_DISABLE);
}

